/************************************************************************************
coordinatetransformationirs
Copyright (c) 2010, International Rescue System Institute
All rights reserved.

Contact us if you use this software for sell.
If you use this software not for sell, you can use this software under BSD license.
See the files LICENSE.TXT and LICENSE-BSD.TXT for more details.                     
************************************************************************************/

//***************************************************************************************//
//* TestInput.cpp							     2009.12.16 *//
//*											*//
//* Test Input RTMプログラム [Ver.1.0.0]						*//
//***************************************************************************************//


// -*- C++ -*-
/*!
 * @file  TestInput.cpp
 * @brief Test Input for Coodinate Transformation Module
 * $Date$
 *
 * $Id$
 */

#include "TestInput.h"

// Module specification
// <rtc-template block="module_spec">
static const char* testinput_spec[] =
  {
    "implementation_id", "TestInput",
    "type_name",         "TestInput",
    "description",       "Test Input for Coordinate Transformation Module",
    "version",           "1.0.0",
    "vendor",            "IRS",
    "category",          "Position",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.01_latitude", "343940.4787",
    "conf.default.02_longitude", "1351256.6632",
    "conf.default.03_19x", "-148161.432",
    "conf.default.04_19y", "80876.888",
    "conf.default.05_robotx", "0",
    "conf.default.06_roboty", "0",

    ""
  };
// </rtc-template>

TestInput::TestInput(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_OutJgd2000PositionOut("OutJgd2000Position", m_OutJgd2000Position),
    m_Out19PositionOut("Out19Position", m_Out19Position),
    m_OutRobotPositionOut("OutRobotPosition", m_OutRobotPosition),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  registerOutPort("OutJgd2000Position", m_OutJgd2000PositionOut);
  registerOutPort("Out19Position", m_Out19PositionOut);
  registerOutPort("OutRobotPosition", m_OutRobotPositionOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

TestInput::~TestInput()
{
}

RTC::ReturnCode_t TestInput::onInitialize()
{
	// <rtc-template block="bind_config">
	// Bind variables and configuration variable
	bindParameter("01_latitude", m_01_latitude, "343940.4787");
	bindParameter("02_longitude", m_02_longitude, "1351256.6632");
	bindParameter("03_19x", m_03_19x, "-148161.432");
	bindParameter("04_19y", m_04_19y, "80876.888");
	bindParameter("05_robotx", m_05_robotx, "0");
	bindParameter("06_roboty", m_06_roboty, "0");
  
	// </rtc-template>

	return RTC::RTC_OK;										// 戻り値 [正常終了]

}

/*
RTC::ReturnCode_t TestInput::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TestInput::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TestInput::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

RTC::ReturnCode_t TestInput::onActivated(RTC::UniqueId ec_id)
{


	//-- [コンフィギュレーション読み込み] ---------------------------------------------------//

	latitude  = m_01_latitude;									// データ格納 [緯度]
	longitude = m_02_longitude;									// データ格納 [経度]
	x         = m_03_19x;										// データ格納 [x座標]
	y         = m_04_19y;										// データ格納 [y座標]
	robot_x   = m_05_robotx;									// データ格納 [x座標]
	robot_y   = m_06_roboty;									// データ格納 [y座標]

	//---------------------------------------------------------------------------------------//


	//-- [デバッグconfiguration] ------------------------------------------------------------//

	#ifndef DEBUG_CONFIGURATION									// デバッグ表示開始

	printf("\n[configuration]\n");									// label
	printf("01_latitude  = %lf\n", latitude);							// latitude
	printf("02_longitude = %lf\n", longitude);							// longitude
	printf("03_19x       = %lf\n", x);								// x
	printf("04_19y       = %lf\n", y);								// y
	printf("05_robotx    = %lf\n", robot_x);							// robot_x
	printf("06_roboty    = %lf\n\n", robot_y);							// robot_y
	fflush(stdout);											// バッファフラッシュ [printf表示用]

	#endif												// デバッグ表示終了

	//---------------------------------------------------------------------------------------//


	//-- [タイムスタンプ格納] ---------------------------------------------------------------//

	timestamp = gettimeofday_sec();									// タイムスタンプ格納
	sec  = (unsigned long int)timestamp;								// タイムスタンプ格納 [sec]
	nsec = (unsigned long int)(fmod(timestamp, 1) * 1000000);					// タイムスタンプ格納 [nsec]

	//---------------------------------------------------------------------------------------//


	//-- [データ出力] -----------------------------------------------------------------------//

	m_OutJgd2000Position.tm.sec    = sec;								// タイムスタンプ格納 [sec]
	m_OutJgd2000Position.tm.nsec   = nsec;								// タイムスタンプ格納 [nsec]
	m_OutJgd2000Position.latitude  = latitude;							// 緯度格納
	m_OutJgd2000Position.longitude = longitude;							// 経度格納

	m_Out19Position.tm.sec  = sec;									// タイムスタンプ格納 [sec]
	m_Out19Position.tm.nsec = nsec;									// タイムスタンプ格納 [nsec]
	m_Out19Position.x       = x;									// x座標格納
	m_Out19Position.y       = y;									// y座標格納

	m_OutRobotPosition.tm.sec  = sec;								// タイムスタンプ格納 [sec]
	m_OutRobotPosition.tm.nsec = nsec;								// タイムスタンプ格納 [nsec]
	m_OutRobotPosition.x       = robot_x;								// x座標格納
	m_OutRobotPosition.y       = robot_y;								// y座標格納

	m_OutJgd2000PositionOut.write();								// データ出力
	m_Out19PositionOut.write();									// データ出力
	m_OutRobotPositionOut.write();									// データ出力

	//---------------------------------------------------------------------------------------//


	return RTC::RTC_OK;										// 戻り値 [正常終了]

}

RTC::ReturnCode_t TestInput::onDeactivated(RTC::UniqueId ec_id)
{

	return RTC::RTC_OK;										// 戻り値 [正常終了]

}

RTC::ReturnCode_t TestInput::onExecute(RTC::UniqueId ec_id)
{

	return RTC::RTC_OK;										// 戻り値 [正常終了]

}

/*
RTC::ReturnCode_t TestInput::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TestInput::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TestInput::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TestInput::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TestInput::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{

  void TestInputInit(RTC::Manager* manager)
  {
    RTC::Properties profile(testinput_spec);
    manager->registerFactory(profile,
                             RTC::Create<TestInput>,
                             RTC::Delete<TestInput>);
  }

};


