/************************************************************************************
coordinatetransformationirs
Copyright (c) 2010, International Rescue System Institute
All rights reserved.

Contact us if you use this software for sell.
If you use this software not for sell, you can use this software under BSD license.
See the files LICENSE.TXT and LICENSE-BSD.TXT for more details.                     
************************************************************************************/

//***********************************************************************************************//
//* CoordinateTransformation.cpp						     2009.12.16 *//
//*												*//
//* Coordinate Transformation RTMプログラム [Ver.1.0.0]						*//
//*												*//
//* コンフィギュレーション									*//
//*	01_inputdatatype	: 入力データタイプ						*//
//*				  [jgd2000, 19, robot]						*//
//*	02_19number		: 座標系番号							*//
//*				  [1, 2, 3, 4, 5, 6, 7, 8, 9, 10,				*//
//*				   11 , 12, 13, 14, 15, 16, 17, 17, 19]				*//
//*	03_19originx		: ロボット座標系原点x座標 [平面直角座標系:m]			*//
//*	04_19originy		: ロボット座標系原点y座標 [平面直角座標系:m]			*//
//*	05_19origintheta	: ロボット座標系原点角度  [平面直角座標系:degree]		*//
//***********************************************************************************************//


// -*- C++ -*-
/*!
 * @file  CoordinateTransformation.cpp
 * @brief Coordinate Transformation
 * $Date$
 *
 * $Id$
 */

#include "CoordinateTransformation.h"

// Module specification
// <rtc-template block="module_spec">
static const char* coordinatetransformation_spec[] =
  {
    "implementation_id", "CoordinateTransformation",
    "type_name",         "CoordinateTransformation",
    "description",       "Coordinate Transformation",
    "version",           "1.0.0",
    "vendor",            "IRS",
    "category",          "Position",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.01_inputdatatype", "jgd2000",
    "conf.default.02_19number", "5",
    "conf.default.03_19originx", "-148161.432",
    "conf.default.04_19originy", "80876.888",
    "conf.default.05_19origintheta", "26.83225571",

    ""
  };
// </rtc-template>

CoordinateTransformation::CoordinateTransformation(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_InJgd2000PositionIn("InJgd2000Position", m_InJgd2000Position),
    m_In19PositionIn("In19Position", m_In19Position),
    m_InRobotPositionIn("InRobotPosition", m_InRobotPosition),
    m_OutJgd2000PositionOut("OutJgd2000Position", m_OutJgd2000Position),
    m_Out19PositionOut("Out19Position", m_Out19Position),
    m_OutRobotPositionOut("OutRobotPosition", m_OutRobotPosition),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("InJgd2000Position", m_InJgd2000PositionIn);
  registerInPort("In19Position", m_In19PositionIn);
  registerInPort("InRobotPosition", m_InRobotPositionIn);
  
  // Set OutPort buffer
  registerOutPort("OutJgd2000Position", m_OutJgd2000PositionOut);
  registerOutPort("Out19Position", m_Out19PositionOut);
  registerOutPort("OutRobotPosition", m_OutRobotPositionOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

CoordinateTransformation::~CoordinateTransformation()
{
}

RTC::ReturnCode_t CoordinateTransformation::onInitialize()
{
	// <rtc-template block="bind_config">
	// Bind variables and configuration variable
	bindParameter("01_inputdatatype", m_01_inputdatatype, "jgd2000");
	bindParameter("02_19number", m_02_19number, "5");
	bindParameter("03_19originx", m_03_19originx, "-148161.432");
	bindParameter("04_19originy", m_04_19originy, "80876.888");
	bindParameter("05_19origintheta", m_05_19origintheta, "26.83225571");
  
	// </rtc-template>


	//-- [初期化] ---------------------------------------------------------------------------//

	trans_init(lng, lat, arc, k1, k2);								// 座標変換初期化関数

	//---------------------------------------------------------------------------------------//


	return RTC::RTC_OK;										// 戻り値 [正常終了]

}

/*
RTC::ReturnCode_t CoordinateTransformation::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CoordinateTransformation::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CoordinateTransformation::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

RTC::ReturnCode_t CoordinateTransformation::onActivated(RTC::UniqueId ec_id)
{


	//-- [初期化] ---------------------------------------------------------------------------//

	sec       = 0;											// タイムスタンプ初期化 [sec]
	nsec      = 0;											// タイムスタンプ初期化 [nsec]
	latitude  = 0.0;										// 緯度初期化
	longitude = 0.0;										// 経度初期化
	x         = 0.0;										// 平面直角座標系x座標初期化
	y         = 0.0;										// 平面直角座標系y座標初期化
	robot_x   = 0.0;										// ロボット座標系x座標初期化
	robot_y   = 0.0;										// ロボット座標系y座標初期化

	//---------------------------------------------------------------------------------------//


	//-- [出力ポート初期化] -----------------------------------------------------------------//

	// 世界測地系座標初期化
	m_OutJgd2000Position.tm.sec    = 0;								// 初期化 [tm]
	m_OutJgd2000Position.tm.nsec   = 0;								// 初期化 [tm]
	m_OutJgd2000Position.latitude  = 0.0;								// 初期化 [latitude]
	m_OutJgd2000Position.longitude = 0.0;								// 初期化 [longitude]

	// 平面直角座標系座標初期化
	m_Out19Position.tm.sec  = 0;									// 初期化 [tm]
	m_Out19Position.tm.nsec = 0;									// 初期化 [tm]
	m_Out19Position.x       = 0.0;									// 初期化 [x]
	m_Out19Position.y       = 0.0;									// 初期化 [y]

	// ロボット座標系座標初期化
	m_OutRobotPosition.tm.sec  = 0;									// 初期化 [tm]
	m_OutRobotPosition.tm.nsec = 0;									// 初期化 [tm]
	m_OutRobotPosition.x       = 0.0;								// 初期化 [x]
	m_OutRobotPosition.y       = 0.0;								// 初期化 [y]

	//---------------------------------------------------------------------------------------//


	//-- [コンフィギュレーション読み込み] ---------------------------------------------------//

	strcpy(inputdatatype, m_01_inputdatatype.c_str());						// データ格納 [タイプ]
	number       = m_02_19number;									// データ格納 [座標系番号]
	origin_x     = m_03_19originx;									// データ格納 [原点x座標]
	origin_y     = m_04_19originy;									// データ格納 [原点y座標]
	origin_theta = m_05_19origintheta;								// データ格納 [原点角度]

	//---------------------------------------------------------------------------------------//


	//-- [デバッグconfiguration] ------------------------------------------------------------//

	#ifndef DEBUG_CONFIGURATION									// デバッグ表示開始

	printf("\n[configuration]\n");									// label
	printf("01_inputdatatype = %s\n", inputdatatype);						// inputdatatype
	printf("02_19number      = %d\n", number);							// number
	printf("03_19originx     = %lf\n", origin_x);							// originx
	printf("04_19originy     = %lf\n", origin_y);							// originy
	printf("05_19origintheta = %lf\n\n", origin_theta);						// origintheta
	fflush(stdout);											// バッファフラッシュ [printf表示用]

	#endif												// デバッグ表示終了

	//---------------------------------------------------------------------------------------//


	//-- [コンフィギュレーションチェック] ---------------------------------------------------//

	// タイプチェック
	if(strcmp("jgd2000", inputdatatype) == 0) type = 0;						// タイプチェック [世界測地系]
	else if(strcmp("19", inputdatatype) == 0) type = 1;						// タイプチェック [平面直角座標系]
	else if(strcmp("robot", inputdatatype) == 0) type = 2;						// タイプチェック [ロボット座標系]
	else{												// 入力エラー [タイプ]

		printf("configuration error:01_inputdatatype = %s\n", inputdatatype);			// エラーメッセージ
		fflush(stdout);										// バッファフラッシュ [printf表示用]
		return RTC::RTC_ERROR;									// 戻り値 [異常終了]

	}

	// 座標系番号チェック
	if((number == 1) || (number == 2) || (number == 3) || (number == 4) || (number == 5) ||
		(number == 6) || (number == 7) || (number == 8) || (number == 9) || (number == 10) ||
		(number == 11) || (number == 12) || (number == 13) || (number == 14) || (number == 15) ||
		(number == 16) || (number == 17) || (number == 18) || (number == 19));			// 座標系番号チェック [1〜19]
	else{

		printf("configuration error:02_19number = %d\n", number);				// エラーメッセージ
		fflush(stdout);										// バッファフラッシュ [printf表示用]
		return RTC::RTC_ERROR;									// 戻り値 [異常終了]

	}

	//---------------------------------------------------------------------------------------//


	return RTC::RTC_OK;										// 戻り値 [正常終了]

}

RTC::ReturnCode_t CoordinateTransformation::onDeactivated(RTC::UniqueId ec_id)
{

	return RTC::RTC_OK;										// 戻り値 [正常終了]

}

RTC::ReturnCode_t CoordinateTransformation::onExecute(RTC::UniqueId ec_id)
{


	//-- [モード] ---------------------------------------------------------------------------//

	if(type == 0){


		//-- [データ更新] ---------------------------------------------------------------//

		if(m_InJgd2000PositionIn.isNew()){							// 更新チェック

			m_InJgd2000PositionIn.read();							// データ読み取り


			//-- [データ格納] -------------------------------------------------------//

			sec       = m_InJgd2000Position.tm.sec;						// タイムスタンプ格納 [sec]
			nsec      = m_InJgd2000Position.tm.nsec;					// タイムスタンプ格納 [nsec]
			latitude  = m_InJgd2000Position.latitude;					// 緯度格納
			longitude = m_InJgd2000Position.longitude;					// 経度格納

			m_OutJgd2000Position.latitude  = latitude;					// 緯度格納
			m_OutJgd2000Position.longitude = longitude;					// 経度格納

			//-----------------------------------------------------------------------//


			//-- [デバッグjgd2000] --------------------------------------------------//

			#ifndef DEBUG_JGD2000								// デバッグ表示開始

			printf("\n[jgd2000]\n");							// label
			printf("tm.sec    = %ld\n", sec);						// sec
			printf("tm.nsec   = %ld\n", nsec);						// nsec
			printf("latitude  = %lf\n", latitude);						// latitude
			printf("longitude = %lf\n\n", longitude);					// longitude
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			#endif										// デバッグ表示終了

			//-----------------------------------------------------------------------//


			//-- [単位変換] ---------------------------------------------------------//

			latitude  = (int)(latitude / 10000)
				+ ((int)(latitude / 100) - (int)(latitude / 10000) * 100) / 60.0
				+ fmod(latitude, 100) / 3600.0;						// 単位変換 [degree]

			longitude = (int)(longitude / 10000)
				+ ((int)(longitude / 100) - (int)(longitude / 10000) * 100) / 60.0
				+ fmod(longitude, 100) / 3600.0;					// 単位変換 [degree]

			latitude  = latitude * M_PI / 180.0;						// 単位変換 [rad]

			longitude = longitude * M_PI / 180.0;						// 単位変換 [rad]

			theta     = - origin_theta * M_PI / 180;					// 単位変換 [rad]

			//-----------------------------------------------------------------------//


			//-- [座標変換] ---------------------------------------------------------//

			trans_19(latitude, longitude, number, &x, &y, lng, lat);			// 平面直角座標系座標変換関数

			robot_x = x - origin_x;								// 平行移動 [軸入替]
			robot_y = y - origin_y;								// 平行移動 [軸入替]

			buf     = cos(theta) * robot_y - sin(theta) * robot_x;				// 回転 [x座標]
			robot_y = sin(theta) * robot_y + cos(theta) * robot_x;				// 回転 [y座標]
			robot_x = buf;									// x座標格納

			//-----------------------------------------------------------------------//


			//-- [デバッグoutput] ---------------------------------------------------//

			#ifndef DEBUG_OUTPUT								// デバッグ表示開始

			printf("\n[output]\n");								// label
			printf("x         = %lf\n", x);							// x
			printf("y         = %lf\n", y);							// y
			printf("robot_x   = %lf\n", robot_x);						// robot_x
			printf("robot_y   = %lf\n\n", robot_y);						// robot_y
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			#endif										// デバッグ表示終了

			//-----------------------------------------------------------------------//


			//-- [データ出力] -------------------------------------------------------//

			m_OutJgd2000Position.tm.sec    = sec;						// タイムスタンプ格納 [sec]
			m_OutJgd2000Position.tm.nsec   = nsec;						// タイムスタンプ格納 [nsec]

			m_Out19Position.tm.sec    = sec;						// タイムスタンプ格納 [sec]
			m_Out19Position.tm.nsec   = nsec;						// タイムスタンプ格納 [nsec]
			m_Out19Position.x         = x;							// x座標格納
			m_Out19Position.y         = y;							// y座標格納

			m_OutRobotPosition.tm.sec    = sec;						// タイムスタンプ格納 [sec]
			m_OutRobotPosition.tm.nsec   = nsec;						// タイムスタンプ格納 [nsec]
			m_OutRobotPosition.x         = robot_x;						// x座標格納
			m_OutRobotPosition.y         = robot_y;						// y座標格納

			m_OutJgd2000PositionOut.write();						// データ出力 [OutJgd2000Position]
			m_Out19PositionOut.write();							// データ出力 [Out19Position]
			m_OutRobotPositionOut.write();							// データ出力 [OutRobotPosition]

			//-----------------------------------------------------------------------//


		}

		//-------------------------------------------------------------------------------//


	}else if(type == 1){


		//-- [データ更新] ---------------------------------------------------------------//

		if(m_In19PositionIn.isNew()){								// 更新チェック

			m_In19PositionIn.read();							// データ読み取り


			//-- [データ格納] -------------------------------------------------------//

			sec  = m_In19Position.tm.sec;							// タイムスタンプ格納 [sec]
			nsec = m_In19Position.tm.nsec;							// タイムスタンプ格納 [nsec]
			x    = m_In19Position.x;							// x座標格納
			y    = m_In19Position.y;							// y座標格納

			m_Out19Position.x         = x;							// x座標格納
			m_Out19Position.y         = y;							// y座標格納

			//-----------------------------------------------------------------------//


			//-- [デバッグ19] -------------------------------------------------------//

			#ifndef DEBUG_19								// デバッグ表示開始

			printf("\n[19]\n");								// label
			printf("tm.sec  = %ld\n", sec);							// sec
			printf("tm.nsec = %ld\n", nsec);						// nsec
			printf("x       = %lf\n", x);							// x
			printf("y       = %lf\n\n", y);							// y
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			#endif										// デバッグ表示終了

			//-----------------------------------------------------------------------//


			//-- [単位変換] ---------------------------------------------------------//

			theta = - origin_theta * M_PI / 180;						// 単位変換 [rad]

			//-----------------------------------------------------------------------//


			//-- [座標変換] ---------------------------------------------------------//

			trans_world(x, y, number, &latitude, &longitude, lng, lat, arc, k1, k2);	// 世界測地系座標変換関数

			robot_x = x - origin_x;								// 平行移動 [軸入替]
			robot_y = y - origin_y;								// 平行移動 [軸入替]

			buf     = cos(theta) * robot_y - sin(theta) * robot_x;				// 回転 [x座標]
			robot_y = sin(theta) * robot_y + cos(theta) * robot_x;				// 回転 [y座標]
			robot_x = buf;									// x座標格納

			//-----------------------------------------------------------------------//


			//-- [単位変換] ---------------------------------------------------------//

			latitude  = latitude * 180.0 / M_PI;						// 単位変換 [degree]

			longitude = longitude * 180.0 / M_PI;						// 単位変換 [degree]

			latitude  = (int)(latitude) * 10000
				+ (int)(fmod(latitude, 1) * 60) * 100
				+ fmod((fmod(latitude, 1) * 60), 1) * 60;				// 単位換算 [DMS]

			longitude = (int)(longitude) * 10000
				+ (int)(fmod(longitude, 1) * 60) * 100
				+ fmod((fmod(longitude, 1) * 60), 1) * 60;				// 単位換算 [DMS]

			//-----------------------------------------------------------------------//


			//-- [デバッグoutput] ---------------------------------------------------//

			#ifndef DEBUG_OUTPUT								// デバッグ表示開始

			printf("\n[output]\n");								// label
			printf("latitude  = %lf\n", latitude);						// latitude
			printf("longitude = %lf\n", longitude);						// longitude
			printf("robot_x   = %lf\n", robot_x);						// robot_x
			printf("robot_y   = %lf\n\n", robot_y);						// robot_y
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			#endif										// デバッグ表示終了

			//-----------------------------------------------------------------------//


			//-- [データ出力] -------------------------------------------------------//

			m_OutJgd2000Position.tm.sec    = sec;						// タイムスタンプ格納 [sec]
			m_OutJgd2000Position.tm.nsec   = nsec;						// タイムスタンプ格納 [nsec]
			m_OutJgd2000Position.latitude  = latitude;					// 緯度格納
			m_OutJgd2000Position.longitude = longitude;					// 経度格納

			m_Out19Position.tm.sec    = sec;						// タイムスタンプ格納 [sec]
			m_Out19Position.tm.nsec   = nsec;						// タイムスタンプ格納 [nsec]

			m_OutRobotPosition.tm.sec    = sec;						// タイムスタンプ格納 [sec]
			m_OutRobotPosition.tm.nsec   = nsec;						// タイムスタンプ格納 [nsec]
			m_OutRobotPosition.x         = robot_x;						// x座標格納
			m_OutRobotPosition.y         = robot_y;						// y座標格納

			m_OutJgd2000PositionOut.write();						// データ出力 [OutJgd2000Position]
			m_Out19PositionOut.write();							// データ出力 [Out19Position]
			m_OutRobotPositionOut.write();							// データ出力 [OutRobotPosition]

			//-----------------------------------------------------------------------//


		}

		//-------------------------------------------------------------------------------//


	}else{


		//-- [データ更新] ---------------------------------------------------------------//

		if(m_InRobotPositionIn.isNew()){							// 更新チェック

			m_InRobotPositionIn.read();							// データ読み取り


			//-- [データ格納] -------------------------------------------------------//

			sec     = m_InRobotPosition.tm.sec;						// タイムスタンプ格納 [sec]
			nsec    = m_InRobotPosition.tm.nsec;						// タイムスタンプ格納 [nsec]
			robot_x = m_InRobotPosition.x;							// x座標格納
			robot_y = m_InRobotPosition.y;							// y座標格納

			m_OutRobotPosition.x         = robot_x;						// x座標格納
			m_OutRobotPosition.y         = robot_y;						// y座標格納

			//-----------------------------------------------------------------------//


			//-- [デバッグrobot] -------------------------------------------------------//

			#ifndef DEBUG_ROBOT								// デバッグ表示開始

			printf("\n[robot]\n");								// label
			printf("tm.sec  = %ld\n", sec);							// sec
			printf("tm.nsec = %ld\n", nsec);						// nsec
			printf("x       = %lf\n", robot_x);						// robot_x
			printf("y       = %lf\n\n", robot_y);						// robot_y
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			#endif										// デバッグ表示終了

			//-----------------------------------------------------------------------//


			//-- [単位変換] ---------------------------------------------------------//

			theta = origin_theta * M_PI / 180;						// 単位変換 [rad]

			//-----------------------------------------------------------------------//


			//-- [座標変換] ---------------------------------------------------------//

			x = cos(theta) * robot_x - sin(theta) * robot_y;				// 回転 [x座標]
			y = sin(theta) * robot_x + cos(theta) * robot_y;				// 回転 [y座標]

			buf = y + origin_x;								// 平行移動 [原点移動]
			y   = x + origin_y;								// 平行移動 [原点移動]
			x   = buf;									// x座標格納

			trans_world(x, y, number, &latitude, &longitude, lng, lat, arc, k1, k2);	// 世界測地系座標変換関数

			//-----------------------------------------------------------------------//


			//-- [単位変換] ---------------------------------------------------------//

			latitude  = latitude * 180.0 / M_PI;						// 単位変換 [degree]

			longitude = longitude * 180.0 / M_PI;						// 単位変換 [degree]

			latitude  = (int)(latitude) * 10000
				+ (int)(fmod(latitude, 1) * 60) * 100
				+ fmod((fmod(latitude, 1) * 60), 1) * 60;				// 単位換算 [DMS]

			longitude = (int)(longitude) * 10000
				+ (int)(fmod(longitude, 1) * 60) * 100
				+ fmod((fmod(longitude, 1) * 60), 1) * 60;				// 単位換算 [DMS]

			//-----------------------------------------------------------------------//


			//-- [デバッグoutput] ---------------------------------------------------//

			#ifndef DEBUG_OUTPUT								// デバッグ表示開始

			printf("\n[output]\n");								// label
			printf("latitude  = %lf\n", latitude);						// latitude
			printf("longitude = %lf\n", longitude);						// longitude
			printf("x         = %lf\n", x);							// x
			printf("y         = %lf\n\n", y);						// y
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			#endif										// デバッグ表示終了

			//-----------------------------------------------------------------------//


			//-- [データ出力] -------------------------------------------------------//

			m_OutJgd2000Position.tm.sec    = sec;						// タイムスタンプ格納 [sec]
			m_OutJgd2000Position.tm.nsec   = nsec;						// タイムスタンプ格納 [nsec]
			m_OutJgd2000Position.latitude  = latitude;					// 緯度格納
			m_OutJgd2000Position.longitude = longitude;					// 経度格納

			m_Out19Position.tm.sec    = sec;						// タイムスタンプ格納 [sec]
			m_Out19Position.tm.nsec   = nsec;						// タイムスタンプ格納 [nsec]
			m_Out19Position.x         = x;							// x座標格納
			m_Out19Position.y         = y;							// y座標格納

			m_OutRobotPosition.tm.sec    = sec;						// タイムスタンプ格納 [sec]
			m_OutRobotPosition.tm.nsec   = nsec;						// タイムスタンプ格納 [nsec]

			m_OutJgd2000PositionOut.write();						// データ出力 [OutJgd2000Position]
			m_Out19PositionOut.write();							// データ出力 [Out19Position]
			m_OutRobotPositionOut.write();							// データ出力 [OutRobotPosition]

			//-----------------------------------------------------------------------//


		}

		//-------------------------------------------------------------------------------//


	}

	//---------------------------------------------------------------------------------------//


	return RTC::RTC_OK;										// 戻り値 [正常終了]

}

/*
RTC::ReturnCode_t CoordinateTransformation::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CoordinateTransformation::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CoordinateTransformation::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CoordinateTransformation::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CoordinateTransformation::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void CoordinateTransformationInit(RTC::Manager* manager)
  {
    RTC::Properties profile(coordinatetransformation_spec);
    manager->registerFactory(profile,
                             RTC::Create<CoordinateTransformation>,
                             RTC::Delete<CoordinateTransformation>);
  }
  
};


