/************************************************************************************
coordinatetransformationirs
Copyright (c) 2010, International Rescue System Institute
All rights reserved.

Contact us if you use this software for sell.
If you use this software not for sell, you can use this software under BSD license.
See the files LICENSE.TXT and LICENSE-BSD.TXT for more details.                     
************************************************************************************/

#ifndef COORDINATETRANSFORMATION_H
#define COORDINATETRANSFORMATION_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>


//-- [ヘッダファイル] ---------------------------------------------------------------------------//
#include "coordinate.h"											// coordinateヘッダファイル読み込み
//-----------------------------------------------------------------------------------------------//


//-- [置換] -------------------------------------------------------------------------------------//
#define DATASIZE 128											// 最大データ文字数
#define DEBUG_CONFIGURATION										// DEBUG定義 [デバッグ用]
#define DEBUG_JGD2000											// DEBUG定義 [デバッグ用]
#define DEBUG_19											// DEBUG定義 [デバッグ用]
#define DEBUG_ROBOT											// DEBUG定義 [デバッグ用]
#define DEBUG_OUTPUT											// DEBUG定義 [デバッグ用]
//-----------------------------------------------------------------------------------------------//


// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">
#include "intellirobotStub.h"

// </rtc-template>

using namespace RTC;

class CoordinateTransformation
  : public RTC::DataFlowComponentBase
{
 public:
  CoordinateTransformation(RTC::Manager* manager);
  ~CoordinateTransformation();

	// The initialize action (on CREATED->ALIVE transition)
	// formaer rtc_init_entry() 
	virtual RTC::ReturnCode_t onInitialize();

  // The finalize action (on ALIVE->END transition)
  // formaer rtc_exiting_entry()
  // virtual RTC::ReturnCode_t onFinalize();

  // The startup action when ExecutionContext startup
  // former rtc_starting_entry()
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  // The shutdown action when ExecutionContext stop
  // former rtc_stopping_entry()
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

	// The activated action (Active state entry action)
	// former rtc_active_entry()
	virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

	// The deactivated action (Active state exit action)
	// former rtc_active_exit()
	virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

	// The execution action that is invoked periodically
	// former rtc_active_do()
	virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  // The aborting action when main logic error occurred.
  // former rtc_aborting_entry()
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  // The error action in ERROR state
  // former rtc_error_do()
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  // The reset action that is invoked resetting
  // This is same but different the former rtc_init_entry()
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  // The state update action that is invoked after onExecute() action
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  // The action that is invoked when execution context's rate is changed
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // Configuration variable declaration
  // <rtc-template block="config_declare">
  std::string m_01_inputdatatype;
  unsigned int m_02_19number;
  double m_03_19originx;
  double m_04_19originy;
  double m_05_19origintheta;
  
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedJgd2000Position m_InJgd2000Position;
  InPort<TimedJgd2000Position> m_InJgd2000PositionIn;
  Timed19Position m_In19Position;
  InPort<Timed19Position> m_In19PositionIn;
  TimedRobotPosition m_InRobotPosition;
  InPort<TimedRobotPosition> m_InRobotPositionIn;
  
  // </rtc-template>

  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedJgd2000Position m_OutJgd2000Position;
  OutPort<TimedJgd2000Position> m_OutJgd2000PositionOut;
  Timed19Position m_Out19Position;
  OutPort<Timed19Position> m_Out19PositionOut;
  TimedRobotPosition m_OutRobotPosition;
  OutPort<TimedRobotPosition> m_OutRobotPositionOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

	private:
		int dummy;


		//-- [変数宣言] -----------------------------------------------------------------//

		// プログラム処理
		int type;										// タイプ格納変数
		unsigned long sec;									// タイムスタンプ格納変数 [sec]
		unsigned long nsec;									// タイムスタンプ格納変数 [nsec]
		double latitude;									// 緯度格納変数
		double longitude;									// 経度格納変数
		double x;										// 平面直角座標系x座標格納変数
		double y;										// 平面直角座標系y座標格納変数
		double robot_x;										// ロボット座標系x座標格納変数
		double robot_y;										// ロボット座標系y座標格納変数
		double buf;										// 一時格納変数

		// コンフィギュレーション
		char inputdatatype[DATASIZE];								// タイプ格納配列
		int number;										// 座標系番号格納変数
		double origin_x;									// 原点x座標格納変数
		double origin_y;									// 原点y座標格納変数
		double origin_theta;									// 原点角度格納変数

		// 座標変換
		double theta;										// 回転角度格納変数 [rad]
		double lng[19];										// 原点経度格納配列 [rad]
		double lat[19];										// 原点緯度格納配列 [rad]
		double arc[19];										// 原点子午線弧長格納配列 [m]
		double k1[19];										// 分子係数格納配列 [ ]
		double k2[19];										// 分母第1項係数格納配列 [ ]

		//-------------------------------------------------------------------------------//


};


extern "C"
{
  void CoordinateTransformationInit(RTC::Manager* manager);
};

#endif // COORDINATETRANSFORMATION_H
